/*  Copyright (C) 2006,2007  Joris Mooij  [joris at jorismooij dot nl]
    Copyright (C) 2002  Martijn Leisink  <martijn@mbfys.kun.nl>
    Radboud University Nijmegen, The Netherlands
    
    This file is part of libDAI.

    libDAI is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    libDAI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libDAI; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef __defined_libdai_boundprop_h
#define __defined_libdai_boundprop_h


extern "C" { 
    #include <glpk.h>
}


#include "lpx.h"
#include "box.h"
#include "factor.h"


namespace libDAI {


    typedef std::set<Factor> Cluster;

    class BoundSet: public std::set<Box> {
        private:
            ::LPX *lpx;
            size_t verbose;

        public:
            BoundSet() : lpx(NULL), verbose(0U) {}
            BoundSet( const std::set<Box> &bs ) : std::set<Box>(bs), verbose(0U) {}
            BoundSet( const Cluster& C ) : verbose(0U) {
                for( Cluster::const_iterator i = C.begin(); i != C.end(); i++ )
                    insert( Box( i->vars() ) );
            }
            void make_lp( const VarSet& sep );
            void delete_lp() { 
                ::lpx_delete_prob(lpx);
            }
            void newbound( Box& b, const Factor& Pjoint, const Factor& Psep ) const;
            BoundSet up( const FactorGraph& fg );
            BoundSet up( const FactorGraph& fg, const Cluster& base );
            size_t Verbose() const { return verbose; }
            void Verbose( size_t new_verbose ) { verbose = new_verbose; }
    };

    void BoundProp_solve( const FactorGraph &net, long maxsize, double tol, size_t maxiter, BoundSet &bounds, size_t verb );
    void BoundProp_setup( const FactorGraph &net, long maxsize, BoundSet &B, Cluster &Bmain, size_t verb );
    void BoundProp_tile( const FactorGraph &net, long maxsize, Cluster &Cex, Cluster &Cin, size_t Verbose = 0 );


}


#endif
