/*  Copyright (C) 2006,2007  Joris Mooij  [joris at jorismooij dot nl]
    Radboud University Nijmegen, The Netherlands
    
    This file is part of libDAI.

    libDAI is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    libDAI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with libDAI; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "factorgraph.h"
#include "exceptions.h"
#include "bounds.h"

#include "tree/tree.hh"



using namespace std;
using namespace libDAI;



int main( int argc, char *argv[] ) {
    if( argc != 4 && argc != 5 && argc != 6 ) {
        cout << "Usage: " << argv[0] << " <in.fg> <out.dot> <mode> [var] [maxnodes]" << endl << endl;
        cout << "Converts a .fg (FactorGraph) file to a .dot (GraphViz) file for" << endl;
        cout << "visualization. The .dot file can be converted to .ps (PostScript) by" << endl;
        cout << "'neato -T ps out.dot > out.ps' or by 'dot -T ps out.dot > out.ps'" << endl << endl;
        cout << "If <mode> == 0, the factor graph itself is rendered." << endl;
        cout << "If <mode> == 1, the bound graph for variable with label [var] is rendered." << endl;
        cout << "If <mode> == 2, the SelfAvoidingWalk tree for variable with label [var]." << endl;
        return 1;
    } else {
        // Read factorgraph
        FactorGraph fg;
        char *infile = argv[1];
        size_t mode = atoi( argv[3] );

        fg.ReadFromFile( infile );

        ostream *os = &cout;
        ofstream outfile;
        if( strcmp( argv[2], "-" ) ) {
            outfile.open( argv[2] );
            if( !outfile.is_open() ) {
                cerr << "Cannot open " << argv[2] << " for writing" << endl;
                return 1;
            }
            os = &outfile;
        }

        try {
            if( mode == 0 ) {
                // output factor graph
                fg.display( *os );
            } else if( mode == 1 || mode == 2 ) {
                // output bound graph
                if( argc != 6 ) {
                    cerr << "Please specify <var> and <maxnodes>" << endl;
                    throw;
                } else {
                    size_t varlabel = atoi( argv[4] );
                    size_t maxnodes = atoi( argv[5] );
                    Var var = Var( varlabel, 0 );
                    BoundTree tr;
                    tr.buildTree( fg, var, maxnodes, mode == 2 );
                    tr.display( *os, fg );
//                    Box box = tr.BoxProp( fg, false, 0 );
//                    cerr << box << endl;
                }
            } else {
                cerr << "Invalid <mode> value." << endl;
                throw;
            }
        } catch (...) {
            if( strcmp( argv[2], "-" ) )
                outfile.close();
            cerr << "An exception occurred." << endl;
            return 1;
        }

        if( strcmp( argv[2], "-" ) )
            outfile.close();
        return 0;
    }
}
